@@ -14,6 +14,8 @@
 #'
 #' - [year-quarter-day][year-quarter-day-arithmetic]
 #'
+#' - [year-week-day][year-week-day-arithmetic]
+#'
 #' - [iso-year-week-day][iso-year-week-day-arithmetic]
 #'
 #' - [year-day][year-day-arithmetic]
@@ -42,6 +44,9 @@
 #' - [date-times (POSIXct / POSIXlt)][posixt-arithmetic]
 #'
 #' @details
+#' `x` and `n` are recycled against each other using
+#' [tidyverse recycling rules][vctrs::vector_recycling_rules].
+#'
 #' Months and years are considered "irregular" because some months have more
 #' days then others (28, 29, 30, or 31), and some years have more days than
 #' others (365 or 366).
@@ -71,162 +76,127 @@
 #' add_months(x, 1)
 NULL
 
-# ------------------------------------------------------------------------------
-
 #' @rdname clock-arithmetic
 #' @export
 add_years <- function(x, n, ...) {
   UseMethod("add_years")
 }
 
-#' @export
-add_years.clock_time_point <- function(x, n, ...) {
-  stop_clock_unsupported_time_point_op("add_years")
-}
-
-#' @export
-add_years.clock_calendar <- function(x, n, ...) {
-  stop_clock_unsupported_calendar_op("add_years")
-}
-
-# ------------------------------------------------------------------------------
-
 #' @rdname clock-arithmetic
 #' @export
 add_quarters <- function(x, n, ...) {
   UseMethod("add_quarters")
 }
 
-#' @export
-add_quarters.clock_time_point <- function(x, n, ...) {
-  stop_clock_unsupported_time_point_op("add_quarters")
-}
-
-#' @export
-add_quarters.clock_calendar <- function(x, n, ...) {
-  stop_clock_unsupported_calendar_op("add_quarters")
-}
-
-# ------------------------------------------------------------------------------
-
 #' @rdname clock-arithmetic
 #' @export
 add_months <- function(x, n, ...) {
   UseMethod("add_months")
 }
 
-#' @export
-add_months.clock_time_point <- function(x, n, ...) {
-  stop_clock_unsupported_time_point_op("add_months")
-}
-
-#' @export
-add_months.clock_calendar <- function(x, n, ...) {
-  stop_clock_unsupported_calendar_op("add_months")
-}
-
-# ------------------------------------------------------------------------------
-
 #' @rdname clock-arithmetic
 #' @export
 add_weeks <- function(x, n, ...) {
   UseMethod("add_weeks")
 }
 
-#' @export
-add_weeks.clock_calendar <- function(x, n, ...) {
-  stop_clock_unsupported_calendar_op("add_weeks")
-}
-
-# ------------------------------------------------------------------------------
-
 #' @rdname clock-arithmetic
 #' @export
 add_days <- function(x, n, ...) {
   UseMethod("add_days")
 }
 
-#' @export
-add_days.clock_calendar <- function(x, n, ...) {
-  stop_clock_unsupported_calendar_op("add_days")
-}
-
-# ------------------------------------------------------------------------------
-
 #' @rdname clock-arithmetic
 #' @export
 add_hours <- function(x, n, ...) {
   UseMethod("add_hours")
 }
 
-#' @export
-add_hours.clock_calendar <- function(x, n, ...) {
-  stop_clock_unsupported_calendar_op("add_hours")
-}
-
-# ------------------------------------------------------------------------------
-
 #' @rdname clock-arithmetic
 #' @export
 add_minutes <- function(x, n, ...) {
   UseMethod("add_minutes")
 }
 
-#' @export
-add_minutes.clock_calendar <- function(x, n, ...) {
-  stop_clock_unsupported_calendar_op("add_minutes")
-}
-
-# ------------------------------------------------------------------------------
-
 #' @rdname clock-arithmetic
 #' @export
 add_seconds <- function(x, n, ...) {
   UseMethod("add_seconds")
 }
 
-#' @export
-add_seconds.clock_calendar <- function(x, n, ...) {
-  stop_clock_unsupported_calendar_op("add_seconds")
-}
-
-# ------------------------------------------------------------------------------
-
 #' @rdname clock-arithmetic
 #' @export
 add_milliseconds <- function(x, n, ...) {
   UseMethod("add_milliseconds")
 }
 
-#' @export
-add_milliseconds.clock_calendar <- function(x, n, ...) {
-  stop_clock_unsupported_calendar_op("add_milliseconds")
-}
-
-# ------------------------------------------------------------------------------
-
 #' @rdname clock-arithmetic
 #' @export
 add_microseconds <- function(x, n, ...) {
   UseMethod("add_microseconds")
 }
 
-#' @export
-add_microseconds.clock_calendar <- function(x, n, ...) {
-  stop_clock_unsupported_calendar_op("add_microseconds")
-}
-
-# ------------------------------------------------------------------------------
-
 #' @rdname clock-arithmetic
 #' @export
 add_nanoseconds <- function(x, n, ...) {
   UseMethod("add_nanoseconds")
 }
 
+# ------------------------------------------------------------------------------
+
 #' @export
-add_nanoseconds.clock_calendar <- function(x, n, ...) {
-  stop_clock_unsupported_calendar_op("add_nanoseconds")
+add_years.default <- function(x, n, ...) {
+  stop_clock_unsupported(x)
+}
+
+#' @export
+add_quarters.default <- function(x, n, ...) {
+  stop_clock_unsupported(x)
+}
+
+#' @export
+add_months.default <- function(x, n, ...) {
+  stop_clock_unsupported(x)
+}
+
+#' @export
+add_weeks.default <- function(x, n, ...) {
+  stop_clock_unsupported(x)
+}
+
+#' @export
+add_days.default <- function(x, n, ...) {
+  stop_clock_unsupported(x)
+}
+
+#' @export
+add_hours.default <- function(x, n, ...) {
+  stop_clock_unsupported(x)
+}
+
+#' @export
+add_minutes.default <- function(x, n, ...) {
+  stop_clock_unsupported(x)
+}
+
+#' @export
+add_seconds.default <- function(x, n, ...) {
+  stop_clock_unsupported(x)
+}
+
+#' @export
+add_milliseconds.default <- function(x, n, ...) {
+  stop_clock_unsupported(x)
+}
+
+#' @export
+add_microseconds.default <- function(x, n, ...) {
+  stop_clock_unsupported(x)
+}
+
+#' @export
+add_nanoseconds.default <- function(x, n, ...) {
+  stop_clock_unsupported(x)
 }
 
 # ------------------------------------------------------------------------------
